'''
Copyright (2015) Sandia Corporation.
Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
the U.S. Government retains certain rights in this software.

Devin Cook <devcook@sandia.gov>

Minimega bindings for Python

**************************************************************************
* THIS FILE IS AUTOMATICALLY GENERATED. DO NOT MODIFY THIS FILE BY HAND. *
**************************************************************************

This API uses a Unix domain socket to communicate with a running instance of
minimega. The protocol is documented here, under "Command Port and the Local
Command Flag":
http://minimega.org/articles/usage.article#TOC_2.2.

This file is automatically generated from the output of "minimega -cli". See
the documentation for genapi.py for details on how to regenerate this file.

This API *should* work for both python2.7 and python3. Please report any issues
to the bug tracker:
https://github.com/sandia-minimega/minimega/issues
'''


import json
import socket
import inspect
from threading import Lock
from collections import namedtuple


# This version is specific to the python API. It is not indicative of the
#  versions of minimega that it can talk with.
__version__ = '2.0a2'

# This is the version of minimega these bindings were created for:
MM_VERSION = 'minimega 93cd5954e895965b804ab9cfb592fd3335d58e51 2016-03-24'

class Error(Exception): pass
class ValidationError(Error): pass


DEFAULT_TIMEOUT = 60

# HAX: python 2/3 hack
try:
    basestring
    def _isstr(obj):
        return isinstance(obj, basestring)
except NameError:
    def _isstr(obj):
        return isinstance(obj, str)


def connect(path):
    '''
    Connect to the minimega instance with UNIX socket at <path> and return
    a new minimega API object.
    '''
    mm = minimega(path)
    if mm.version()[0]['Response'] != MM_VERSION:
        print('WARNING: API was built using a different version of minimega')
    return mm

Command = namedtuple('Command', ['cmd', 'args'])

class SubCommand:
    def __init__(self, mm):
        self.mm = mm


def serializeCommand(command):
    '''
    Returns a string representation of the Command object passed.
    '''
    if not isinstance(command, Command):
        raise TypeError('command must be an instance of Command')
    args = ' '.join(map(str, command.args))
    return '{} {}'.format(command.cmd.shared_prefix, args)


class minimega:
    '''
    This class communicates with a running instance of minimega using a Unix
    domain socket. The protocol is specified here:
    https://code.google.com/p/minimega/wiki/UserGuide#Command_Port_and_the_Local_Command_Flag

    Each minimega command can be called from this object, and the response will
    be returned unless an Exception is thrown.
    '''

    def __init__(self, path, timeout=None):
        '''Connects to the minimega instance with Unix socket at <path>.'''
        self.mm = self
        self._linkCommands(self)
        self.lock = Lock()
        self.moreResponses = False
        self.expectStreamingResponses = False
        self._debug = False
        self._path = path
        self._timeout = timeout
        self._socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self._socket.settimeout(timeout if timeout != None else DEFAULT_TIMEOUT)
        self._socket.connect(path)

    def _linkCommands(self, subcommand):
        '''
        Recursively instantiate all subcommand classes and link them with the
        current instance of mm.
        '''
        for name, cmd in inspect.getmembers(subcommand,
                                            predicate=inspect.isclass):
            if name.startswith('_'):
                continue
            setattr(subcommand, name, cmd(self))
            self._linkCommands(getattr(subcommand, name))

    def _reconnect(self):
        try:
            self._socket.close()
        except:
            pass

        self.__init__(self._path, self._timeout)

    def _getResponse(self):
        msg = ''
        more = self._socket.recv(1).decode('utf-8')
        response = None
        while response is None and more:
            msg += more
            if more != '}':
                #want to read a full JSON object, and not run json.loads for
                # every byte read
                more = self._socket.recv(1).decode('utf-8')
                continue

            try:
                response = json.loads(msg)
            except ValueError as e:
                if self._debug:
                    print(e)
                more = self._socket.recv(1).decode('utf-8')

        if not msg:
            raise Error('Expected response, socket closed')

        if self._debug:
            print('[debug] response: ' + msg)
        if response['Resp'] and response['Resp'][0]['Error']:
            raise Error(response['Resp'][0]['Error'])

        return response['Resp'], response['More']

    def _send(self, cmd, *args):
        if self.moreResponses:
            raise Error('more responses to be read from last command')

        msg = json.dumps({'Original': cmd + ' ' + ' '.join(map(str, args))},
                         separators=(',', ':'))
        if self._debug:
            print('[debug] sending cmd: ' + msg)
        with self.lock:
            if len(msg) != self._socket.send(msg.encode('utf-8')):
                raise Error('failed to write message to minimega')
            if self.expectStreamingResponses:
                #if they want to stream the responses, we can just return
                self.moreResponses = True
                return

            response, self.moreResponses = self._getResponse()

        #return first/only response
        return response

    def streamResponses(self):
        with self.lock:
            while self.moreResponses:
                #more flag was set, return a generator
                response, self.moreResponses = self._getResponse()
                yield response

    #GENERATED COMMANDS:

    
    def bridge(self, *args):
        '''When called with no arguments, display information about all managed bridges.

To add a trunk interface to a specific bridge, use 'bridge trunk'. For example,
to add interface bar to bridge foo:

	bridge trunk foo bar

To create a vxlan or GRE tunnel to another bridge, use 'bridge tunnel'. For example, to create a vxlan tunnel to another bridge with IP 10.0.0.1:

	bridge tunnel vxlan, mega_bridge 10.0.0.1'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'trunk', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'interface', u'type': 'stringItem', 'optional': False, u'key': u'interface'}], [{u'text': u'notrunk', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'interface', u'type': 'stringItem', 'optional': False, u'key': u'interface'}], [{u'text': u'tunnel', u'type': 'literalItem', 'optional': False}, {u'text': u'vxlan,gre', u'type': 'choiceItem', u'options': [u'vxlan', u'gre'], 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'remote ip', u'type': 'stringItem', 'optional': False, u'key': u'remote'}], [{u'text': u'notunnel', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'interface', u'type': 'stringItem', 'optional': False, u'key': u'interface'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('bridge', *args)
        raise ValidationError('could not understand command', args)
    bridge.shared_prefix = 'bridge'
    

    
    def tap(self, *args):
        '''Control host taps on a named vlan for communicating between a host and any VMs
on that vlan.

Calling tap with no arguments will list all created taps.

To create a tap on a particular vlan, invoke tap with the create command:

	tap create <vlan>

For example, to create a host tap with ip and netmask 10.0.0.1/24 on VLAN 5:

	tap create 5 ip 10.0.0.1/24

Optionally, you can specify the bridge to create the host tap on:

	tap create <vlan> bridge <bridge> ip <ip>

You can also optionally specify the tap name, otherwise the tap will be in the
form of mega_tapX.

Additionally, you can bring the tap up with DHCP by using "dhcp" instead of a
ip/netmask:

	tap create 5 dhcp

To delete a host tap, use the delete command and tap name from the tap list:

	tap delete <id>

To delete all host taps, use id all, or 'clear tap':

	tap delete all'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'name', u'type': 'literalItem', 'optional': False}, {u'text': u'tap name', u'type': 'stringItem', 'optional': False, u'key': u'tap'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'dhcp,', u'type': 'choiceItem', u'options': [u'dhcp'], 'optional': False}, {u'text': u'tap name', u'type': 'stringItem', 'optional': True, u'key': u'tap'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'ip', u'type': 'literalItem', 'optional': False}, {u'text': u'ip', u'type': 'stringItem', 'optional': False, u'key': u'ip'}, {u'text': u'tap name', u'type': 'stringItem', 'optional': True, u'key': u'tap'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'bridge', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'bridge', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'name', u'type': 'literalItem', 'optional': False}, {u'text': u'tap name', u'type': 'stringItem', 'optional': True, u'key': u'tap'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'bridge', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'dhcp,', u'type': 'choiceItem', u'options': [u'dhcp'], 'optional': False}, {u'text': u'tap name', u'type': 'stringItem', 'optional': True, u'key': u'tap'}], [{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}, {u'text': u'bridge', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'ip', u'type': 'literalItem', 'optional': False}, {u'text': u'ip', u'type': 'stringItem', 'optional': False, u'key': u'ip'}, {u'text': u'tap name', u'type': 'stringItem', 'optional': True, u'key': u'tap'}], [{u'text': u'delete,', u'type': 'choiceItem', u'options': [u'delete'], 'optional': False}, {u'text': u'id or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('tap', *args)
        raise ValidationError('could not understand command', args)
    tap.shared_prefix = 'tap'
    

    
    def cc(self, *args):
        '''Command and control for virtual machines running the miniccc client. Commands
may include regular commands, backgrounded commands, and any number of sent
and/or received files. Commands will be executed in command creation order. For
example, to send a file 'foo' and display the contents on a remote VM:

	cc send foo
	cc exec cat foo

Files to be sent must be in the filepath directory, as set by the -filepath
flag when launching minimega.

Responses are organized in a structure within <filepath>/miniccc_responses, and
include subdirectories for each client response named by the client's UUID.
Responses can also be displayed on the command line with the 'responses'
command.

Filters may be set to limit which clients may execute a posted command.  For
example, to filter on VMs that are running windows and have a specific IP.

	cc filter os=windows ip=10.0.0.1

For more documentation, see the article "Command and Control API Tutorial".'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'clients,', u'type': 'choiceItem', u'options': [u'clients'], 'optional': False}], [{u'text': u'prefix,', u'type': 'choiceItem', u'options': [u'prefix'], 'optional': False}, {u'text': u'prefix', u'type': 'stringItem', 'optional': True, u'key': u'prefix'}], [{u'text': u'send,', u'type': 'choiceItem', u'options': [u'send'], 'optional': False}, {u'text': u'file', u'type': 'listItem', 'optional': False, u'key': u'file'}], [{u'text': u'recv,', u'type': 'choiceItem', u'options': [u'recv'], 'optional': False}, {u'text': u'file', u'type': 'listItem', 'optional': False, u'key': u'file'}], [{u'text': u'exec,', u'type': 'choiceItem', u'options': [u'exec'], 'optional': False}, {u'text': u'command', u'type': 'listItem', 'optional': False, u'key': u'command'}], [{u'text': u'background,', u'type': 'choiceItem', u'options': [u'background'], 'optional': False}, {u'text': u'command', u'type': 'listItem', 'optional': False, u'key': u'command'}], [{u'text': u'process,', u'type': 'choiceItem', u'options': [u'process'], 'optional': False}, {u'text': u'list,', u'type': 'choiceItem', u'options': [u'list'], 'optional': False}, {u'text': u'vm id, name, uuid or all', u'type': 'stringItem', 'optional': False, u'key': u'vm'}], [{u'text': u'process,', u'type': 'choiceItem', u'options': [u'process'], 'optional': False}, {u'text': u'kill,', u'type': 'choiceItem', u'options': [u'kill'], 'optional': False}, {u'text': u'pid', u'type': 'stringItem', 'optional': False, u'key': u'pid'}], [{u'text': u'commands,', u'type': 'choiceItem', u'options': [u'commands'], 'optional': False}], [{u'text': u'filter,', u'type': 'choiceItem', u'options': [u'filter'], 'optional': False}, {u'text': u'filter', u'type': 'listItem', 'optional': True, u'key': u'filter'}], [{u'text': u'responses,', u'type': 'choiceItem', u'options': [u'responses'], 'optional': False}, {u'text': u'id or prefix or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}, {u'text': u'raw,', u'type': 'choiceItem', u'options': [u'raw'], 'optional': True}], [{u'text': u'tunnel,', u'type': 'choiceItem', u'options': [u'tunnel'], 'optional': False}, {u'text': u'uuid', u'type': 'stringItem', 'optional': False, u'key': u'uuid'}, {u'text': u'src port', u'type': 'stringItem', 'optional': False, u'key': u'src'}, {u'text': u'host', u'type': 'stringItem', 'optional': False, u'key': u'host'}, {u'text': u'dst port', u'type': 'stringItem', 'optional': False, u'key': u'dst'}], [{u'text': u'rtunnel,', u'type': 'choiceItem', u'options': [u'rtunnel'], 'optional': False}, {u'text': u'src port', u'type': 'stringItem', 'optional': False, u'key': u'src'}, {u'text': u'host', u'type': 'stringItem', 'optional': False, u'key': u'host'}, {u'text': u'dst port', u'type': 'stringItem', 'optional': False, u'key': u'dst'}], [{u'text': u'delete,', u'type': 'choiceItem', u'options': [u'delete'], 'optional': False}, {u'text': u'command,', u'type': 'choiceItem', u'options': [u'command'], 'optional': False}, {u'text': u'id or prefix or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}], [{u'text': u'delete,', u'type': 'choiceItem', u'options': [u'delete'], 'optional': False}, {u'text': u'response,', u'type': 'choiceItem', u'options': [u'response'], 'optional': False}, {u'text': u'id or prefix or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('cc', *args)
        raise ValidationError('could not understand command', args)
    cc.shared_prefix = 'cc'
    

    
    class vm(SubCommand):
        
    
        def info(self, *args):
            '''Print information about VMs in tabular form. The .filter and .columns commands
can be used to subselect a set of rows and/or columns. See the help pages for
.filter and .columns, respectively, for their usage. Columns returned by VM
info include:

- host       : the host that the VM is running on
- id         : the VM ID, as an integer
- name       : the VM name, if it exists
- state      : one of (building, running, paused, quit, error)
- type       : one of (kvm)
- vcpus      : the number of allocated CPUs
- memory     : allocated memory, in megabytes
- vlan       : vlan, as an integer
- bridge     : bridge name
- tap        : tap name
- mac        : mac address
- ip         : IPv4 address
- ip6        : IPv6 address
- bandwidth  : stats regarding bandwidth usage
- tags       : any additional information attached to the VM
- uuid       : QEMU system uuid
- cc_active  : whether cc is active

Additional fields are available for KVM-based VMs:

- append     : kernel command line string
- cdrom      : cdrom image
- disk       : disk image
- kernel     : kernel image
- initrd     : initrd image
- migrate    : qemu migration image

Additional fields are available for container-based VMs:

- init	     : process to invoke as init
- preinit    : process to invoke at container launch before isolation
- filesystem : root filesystem for the container

Examples:

Display a list of all IPs for all VMs:
	.columns ip,ip6 vm info

Display information about all VMs:
	vm info'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm info', *args)
            raise ValidationError('could not understand command', args)
        info.shared_prefix = 'vm info'
    

    
        def screenshot(self, *args):
            '''Take a screenshot of the framebuffer of a running VM. The screenshot is saved
in PNG format as "screenshot.png" in the VM's runtime directory (by default
/tmp/minimega/<vm id>/screenshot.png).

An optional argument sets the maximum dimensions in pixels, while keeping the
aspect ratio. For example, to set either maximum dimension of the output image
to 100 pixels:

	vm screenshot foo 100

The screenshot can be saved elsewhere like this:

        vm screenshot foo file /tmp/foo.png

You can also specify the maximum dimension:

        vm screenshot foo file /tmp/foo.png 100'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'maximum dimension', u'type': 'stringItem', 'optional': True, u'key': u'maximum'}], [{u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'file', u'type': 'literalItem', 'optional': False}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}, {u'text': u'maximum dimension', u'type': 'stringItem', 'optional': True, u'key': u'maximum'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm screenshot', *args)
            raise ValidationError('could not understand command', args)
        screenshot.shared_prefix = 'vm screenshot'
    

    
        def launch(self, *args):
            '''Launch virtual machines in a paused state, using the parameters defined leading
up to the launch command. Any changes to the VM parameters after launching will
have no effect on launched VMs.

When you launch a VM, you supply the type of VM in the launch command.
Currently, the supported VM types are:

- kvm : QEMU-based vms

If you supply a name instead of a number of VMs, one VM with that name will be
launched. You may also supply a range expression to launch VMs with a specific
naming scheme:

	vm launch foo[0-9]

Note: VM names cannot be integers or reserved words (e.g. "all").

The optional 'noblock' suffix forces minimega to return control of the command
line immediately instead of waiting on potential errors from launching the
VM(s). The user must check logs or error states from vm info.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'kvm,container', u'type': 'choiceItem', u'options': [u'kvm', u'container'], 'optional': False}, {u'text': u'name or count', u'type': 'stringItem', 'optional': False, u'key': u'name'}, {u'text': u'noblock,', u'type': 'choiceItem', u'options': [u'noblock'], 'optional': True}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm launch', *args)
            raise ValidationError('could not understand command', args)
        launch.shared_prefix = 'vm launch'
    

    
        def migrate(self, *args):
            '''Migrate runtime state of a VM to disk, which can later be booted with vm config
migrate.

Migration files are written to the files directory as specified with -filepath.
On success, a call to migrate a VM will return immediately. You can check the
status of in-flight migrations by invoking vm migrate with no arguments.'''
            args = list(args)
            #validate the args
            candidates = [[], [{u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm migrate', *args)
            raise ValidationError('could not understand command', args)
        migrate.shared_prefix = 'vm migrate'
    

    
        def qmp(self, *args):
            '''Issue a JSON-encoded QMP command. This is a convenience function for accessing
the QMP socket of a VM via minimega. vm qmp takes two arguments, a VM ID or
name, and a JSON string, and returns the JSON encoded response. For example:

	vm qmp 0 '{ "execute": "query-status" }'
	{"return":{"running":false,"singlestep":false,"status":"prelaunch"}}'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'qmp command', u'type': 'stringItem', 'optional': False, u'key': u'qmp'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm qmp', *args)
            raise ValidationError('could not understand command', args)
        qmp.shared_prefix = 'vm qmp'
    

    
        def stop(self, *args):
            '''Stop one or more running virtual machines. See "vm start" for a full
description of allowable targets.

Calling stop will put VMs in a paused state. Use "vm start" to restart them.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'target', u'type': 'stringItem', 'optional': False, u'key': u'target'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm stop', *args)
            raise ValidationError('could not understand command', args)
        stop.shared_prefix = 'vm stop'
    

    
        def hotplug(self, *args):
            '''Add and remove USB drives to a launched VM.

To view currently attached media, call vm hotplug with the 'show' argument and
a VM ID or name. To add a device, use the 'add' argument followed by the VM ID
or name, and the name of the file to add. For example, to add foo.img to VM 5:

	vm hotplug add 5 foo.img

The add command will assign a disk ID, shown in vm hotplug show. To remove
media, use the 'remove' argument with the VM ID and the disk ID. For example,
to remove the drive added above, named 0:

	vm hotplug remove 5 0

To remove all hotplug devices, use ID "all" for the disk ID.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'show,', u'type': 'choiceItem', u'options': [u'show'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}], [{u'text': u'add,', u'type': 'choiceItem', u'options': [u'add'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}], [{u'text': u'remove,', u'type': 'choiceItem', u'options': [u'remove'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'disk id or all', u'type': 'stringItem', 'optional': False, u'key': u'disk'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm hotplug', *args)
            raise ValidationError('could not understand command', args)
        hotplug.shared_prefix = 'vm hotplug'
    

    
        def start(self, *args):
            '''Start one or more paused virtual machines. VMs may be selected by name, ID, range, or
wildcard. For example,

To start vm foo:

		vm start foo

To start vms foo and bar:

		vm start foo,bar

To start vms foo0, foo1, foo2, and foo5:

		vm start foo[0-2,5]

VMs can also be specified by ID, such as:

		vm start 0

Or, a range of IDs:

		vm start [2-4,6]

There is also a wildcard (all) which allows the user to specify all VMs:

		vm start all

Note that including the wildcard in a list of VMs results in the wildcard
behavior (although a message will be logged).

Calling "vm start" on a specific list of VMs will cause them to be started if
they are in the building, paused, quit, or error states. When used with the
wildcard, only vms in the building or paused state will be started.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'target', u'type': 'stringItem', 'optional': False, u'key': u'target'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm start', *args)
            raise ValidationError('could not understand command', args)
        start.shared_prefix = 'vm start'
    

    
        def tag(self, *args):
            '''Display or set a tag for one or more virtual machines. See "vm start" for a
full description of allowable targets.

Tags are key-value pairs. A VM can have any number of tags associated with it.
They can be used to attach additional information to a virtual machine, for
example specifying a VM "group", or the correct rendering color for some
external visualization tool.

To set a tag "foo" to "bar" for VM 2:

        vm tag 2 foo bar

To read a tag:

        vm tag <target> <key or all>'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'target', u'type': 'stringItem', 'optional': False, u'key': u'target'}, {u'text': u'key or all', u'type': 'stringItem', 'optional': True, u'key': u'key'}], [{u'text': u'target', u'type': 'stringItem', 'optional': False, u'key': u'target'}, {u'text': u'key', u'type': 'stringItem', 'optional': False, u'key': u'key'}, {u'text': u'value', u'type': 'stringItem', 'optional': False, u'key': u'value'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm tag', *args)
            raise ValidationError('could not understand command', args)
        tag.shared_prefix = 'vm tag'
    

    
        def kill(self, *args):
            '''Kill one or more running virtual machines. See "vm start" for a full
description of allowable targets.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'target', u'type': 'stringItem', 'optional': False, u'key': u'target'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm kill', *args)
            raise ValidationError('could not understand command', args)
        kill.shared_prefix = 'vm kill'
    

    
        def flush(self, *args):
            '''Discard information about VMs that have either quit or encountered an error.
This will remove any VMs with a state of "quit" or "error" from vm info. Names
of VMs that have been flushed may be reused.'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm flush', *args)
            raise ValidationError('could not understand command', args)
        flush.shared_prefix = 'vm flush'
    

    
        def cdrom(self, *args):
            '''Eject or change an active VM's cdrom image.

Eject VM 0's cdrom:

        vm cdrom eject 0

Eject all VM cdroms:

        vm cdrom eject all

Change a VM to use a new ISO:

        vm cdrom change 0 /tmp/debian.iso

"vm cdrom change" implies that the current ISO will be ejected.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'eject,', u'type': 'choiceItem', u'options': [u'eject'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}], [{u'text': u'change,', u'type': 'choiceItem', u'options': [u'change'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'path', u'type': 'stringItem', 'optional': False, u'key': u'path'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm cdrom', *args)
            raise ValidationError('could not understand command', args)
        cdrom.shared_prefix = 'vm cdrom'
    

    
        def net(self, *args):
            '''Disconnect or move existing network connections on a running VM.

Network connections are indicated by their position in vm net (same order in vm
info) and are zero indexed. For example, to disconnect the first network
connection from a VM named vm-0 with 4 network connections:

	vm net disconnect vm-0 0

To disconnect the second connection:

	vm net disconnect vm-0 1

To move a connection, specify the new VLAN tag and bridge:

	vm net <vm name or id> 0 bridgeX 100'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'connect,', u'type': 'choiceItem', u'options': [u'connect'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'tap position', u'type': 'stringItem', 'optional': False, u'key': u'tap'}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'vlan', u'type': 'stringItem', 'optional': False, u'key': u'vlan'}], [{u'text': u'disconnect,', u'type': 'choiceItem', u'options': [u'disconnect'], 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'tap position', u'type': 'stringItem', 'optional': False, u'key': u'tap'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm net', *args)
            raise ValidationError('could not understand command', args)
        net.shared_prefix = 'vm net'
    

    
        def save(self, *args):
            '''Saves the configuration of a running virtual machine or set of virtual machines
so that it/they can be restarted/recovered later, such as after a system crash.

This command does not store the state of the virtual machine itself, only its
launch configuration.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'name', u'type': 'stringItem', 'optional': False, u'key': u'name'}, {u'text': u'vm id or name or all', u'type': 'listItem', 'optional': False, u'key': u'vm'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('vm save', *args)
            raise ValidationError('could not understand command', args)
        save.shared_prefix = 'vm save'
    

    
        class config(SubCommand):
        
    
            def kernel(self, *args):
                '''Attach a kernel image to a VM. If set, QEMU will boot from this image instead
of any disk image.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'path to kernel', u'type': 'stringItem', 'optional': True, u'key': u'path'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config kernel', *args)
                raise ValidationError('could not understand command', args)
            kernel.shared_prefix = 'vm config kernel'
    

    
            def migrate(self, *args):
                '''Assign a migration image, generated by a previously saved VM to boot with.
Migration images should be booted with a kernel/initrd, disk, or cdrom. Use 'vm
migrate' to generate migration images from running VMs.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'path to migration image', u'type': 'stringItem', 'optional': True, u'key': u'path'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config migrate', *args)
                raise ValidationError('could not understand command', args)
            migrate.shared_prefix = 'vm config migrate'
    

    
            def fifo(self, *args):
                '''Set the number of named pipes to include in the container for container-host
communication. Named pipes will appear on the host in the instance directory
for the container as fifoN, and on the container as /dev/fifos/fifoN.

Fifos are created using mkfifo() and have all of the same usage constraints.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'number', u'type': 'stringItem', 'optional': True, u'key': u'number'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config fifo', *args)
                raise ValidationError('could not understand command', args)
            fifo.shared_prefix = 'vm config fifo'
    

    
            def initrd(self, *args):
                '''Attach an initrd image to a VM. Passed along with the kernel image at boot
time.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'path to initrd', u'type': 'stringItem', 'optional': True, u'key': u'path'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config initrd', *args)
                raise ValidationError('could not understand command', args)
            initrd.shared_prefix = 'vm config initrd'
    

    
            def tag(self, *args):
                '''Set tags in the same manner as "vm tag". These tags will apply to all newly
launched VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'key', u'type': 'stringItem', 'optional': True, u'key': u'key'}], [{u'text': u'key', u'type': 'stringItem', 'optional': False, u'key': u'key'}, {u'text': u'value', u'type': 'stringItem', 'optional': False, u'key': u'value'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config tag', *args)
                raise ValidationError('could not understand command', args)
            tag.shared_prefix = 'vm config tag'
    

    
            def serial(self, *args):
                '''Specify the serial ports that will be created for the VM to use.
Serial ports specified will be mapped to the VM's /dev/ttySX device, where X
refers to the connected unix socket on the host at
$minimega_runtime/<vm_id>/serialX.

Examples:

To display current serial ports:
  vm config serial

To create three serial ports:
  vm config serial 3

Note: Whereas modern versions of Windows support up to 256 COM ports, Linux
typically only supports up to four serial devices. To use more, make sure to
pass "8250.n_uarts = 4" to the guest Linux kernel at boot. Replace 4 with
another number.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'number of serial ports', u'type': 'stringItem', 'optional': True, u'key': u'number'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config serial', *args)
                raise ValidationError('could not understand command', args)
            serial.shared_prefix = 'vm config serial'
    

    
            def disk(self, *args):
                '''Attach one or more disks to a vm. Any disk image supported by QEMU is a valid
parameter. Disk images launched in snapshot mode may safely be used for
multiple VMs.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'path to disk image', u'type': 'listItem', 'optional': True, u'key': u'path'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config disk', *args)
                raise ValidationError('could not understand command', args)
            disk.shared_prefix = 'vm config disk'
    

    
            def qemu(self, *args):
                '''Set the QEMU process to invoke. Relative paths are ok.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'path to qemu', u'type': 'stringItem', 'optional': True, u'key': u'path'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config qemu', *args)
                raise ValidationError('could not understand command', args)
            qemu.shared_prefix = 'vm config qemu'
    

    
            def append(self, *args):
                '''Add an append string to a kernel set with vm kernel. Setting vm append without
using vm kernel will result in an error.

For example, to set a static IP for a linux VM:

	vm config append ip=10.0.0.5 gateway=10.0.0.1 netmask=255.255.255.0 dns=10.10.10.10

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'arg', u'type': 'listItem', 'optional': True, u'key': u'arg'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config append', *args)
                raise ValidationError('could not understand command', args)
            append.shared_prefix = 'vm config append'
    

    
            def uuid(self, *args):
                '''Set the UUID for a virtual machine. If not set, minimega will create a random
one when the VM is launched.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'uuid', u'type': 'stringItem', 'optional': True, u'key': u'uuid'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config uuid', *args)
                raise ValidationError('could not understand command', args)
            uuid.shared_prefix = 'vm config uuid'
    

    
            def qemuappend(self, *args):
                '''Add additional arguments to be passed to the QEMU instance. For example:

	vm config qemu-append -serial tcp:localhost:4001

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'argument', u'type': 'listItem', 'optional': True, u'key': u'argument'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config qemu-append', *args)
                raise ValidationError('could not understand command', args)
            qemuappend.shared_prefix = 'vm config qemu-append'
    

    
            def hostname(self, *args):
                '''Set a hostname for a container before launching the init program. If not set,
the hostname will be that of the physical host. The hostname can also be set by
the init program or other root process in the container.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'hostname', u'type': 'stringItem', 'optional': True, u'key': u'hostname'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config hostname', *args)
                raise ValidationError('could not understand command', args)
            hostname.shared_prefix = 'vm config hostname'
    

    
            def init(self, *args):
                '''Set the init program and args to exec into upon container launch. This will be
PID 1 in the container.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'init', u'type': 'listItem', 'optional': True, u'key': u'init'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config init', *args)
                raise ValidationError('could not understand command', args)
            init.shared_prefix = 'vm config init'
    

    
            def memory(self, *args):
                '''Set the amount of physical memory to allocate in megabytes.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'memory in megabytes', u'type': 'stringItem', 'optional': True, u'key': u'memory'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config memory', *args)
                raise ValidationError('could not understand command', args)
            memory.shared_prefix = 'vm config memory'
    

    
            def net(self, *args):
                '''Specify the network(s) that the VM is a member of by VLAN. A corresponding VLAN
will be created for each network. Optionally, you may specify the bridge the
interface will be connected on. If the bridge name is omitted, minimega will
use the default 'mega_bridge'. You can also optionally specify the mac address
of the interface to connect to that network. If not specifed, the mac address
will be randomly generated. Additionally, you can optionally specify a driver
for qemu to use. By default, e1000 is used.

Examples:

To connect a VM to VLANs 1 and 5:

	vm config net 1 5

To connect a VM to VLANs 100, 101, and 102 with specific mac addresses:

	vm config net 100,00:00:00:00:00:00 101,00:00:00:00:01:00 102,00:00:00:00:02:00

To connect a VM to VLAN 1 on bridge0 and VLAN 2 on bridge1:

	vm config net bridge0,1 bridge1,2

To connect a VM to VLAN 100 on bridge0 with a specific mac:

	vm config net bridge0,100,00:11:22:33:44:55

To specify a specific driver, such as i82559c:

	vm config net 100,i82559c

Calling vm net with no parameters will list the current networks for this VM.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'netspec', u'type': 'listItem', 'optional': True, u'key': u'netspec'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config net', *args)
                raise ValidationError('could not understand command', args)
            net.shared_prefix = 'vm config net'
    

    
            def cdrom(self, *args):
                '''Attach a cdrom to a VM. When using a cdrom, it will automatically be set to be
the boot device.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'path to cdrom image', u'type': 'stringItem', 'optional': True, u'key': u'path'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config cdrom', *args)
                raise ValidationError('could not understand command', args)
            cdrom.shared_prefix = 'vm config cdrom'
    

    
            def qemuoverride(self, *args):
                '''Override parts of the QEMU launch string by supplying a string to match, and a
replacement string.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[], [{u'text': u'add', u'type': 'literalItem', 'optional': False}, {u'text': u'match', u'type': 'stringItem', 'optional': False, u'key': u'match'}, {u'text': u'replacement', u'type': 'stringItem', 'optional': False, u'key': u'replacement'}], [{u'text': u'delete', u'type': 'literalItem', 'optional': False}, {u'text': u'id or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config qemu-override', *args)
                raise ValidationError('could not understand command', args)
            qemuoverride.shared_prefix = 'vm config qemu-override'
    

    
            def cpu(self, *args):
                '''Set the virtual CPU architecture.

By default, set to 'host' which matches the host architecture. See 'kvm -cpu
help' for a list of architectures available for your version of kvm.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'cpu', u'type': 'stringItem', 'optional': True, u'key': u'cpu'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config cpu', *args)
                raise ValidationError('could not understand command', args)
            cpu.shared_prefix = 'vm config cpu'
    

    
            def vcpus(self, *args):
                '''Set the number of virtual CPUs to allocate for a VM.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'number of CPUs', u'type': 'stringItem', 'optional': True, u'key': u'number'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config vcpus', *args)
                raise ValidationError('could not understand command', args)
            vcpus.shared_prefix = 'vm config vcpus'
    

    
            def snapshot(self, *args):
                '''Enable or disable snapshot mode when using disk images. When enabled, disks
images will be loaded in memory when run and changes will not be saved. This
allows a single disk image to be used for many VMs.

Note: this configuration only applies to KVM-based VMs.'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'true,false', u'type': 'choiceItem', u'options': [u'true', u'false'], 'optional': True}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config snapshot', *args)
                raise ValidationError('could not understand command', args)
            snapshot.shared_prefix = 'vm config snapshot'
    

    
            def filesystem(self, *args):
                '''Set the filesystem to use for launching a container. This should be a root
filesystem for a linux distribution (containing /dev, /proc, /sys, etc.)'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'filesystem', u'type': 'stringItem', 'optional': True, u'key': u'filesystem'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config filesystem', *args)
                raise ValidationError('could not understand command', args)
            filesystem.shared_prefix = 'vm config filesystem'
    

    
            def preinit(self, *args):
                '''Containers start in a highly restricted environment. vm config preinit allows
running processes before isolation mechanisms are enabled. This occurs when the
vm is launched and before the vm is put in the building state. preinit
processes must finish before the vm will be allowed to start.

Specifically, the preinit command will be run after entering namespaces, and
mounting dependent filesystems, but before cgroups and root capabilities are
set, and before entering the chroot. This means that the preinit command is run
as root and can control the host.

For example, to run a script that enables ip forwarding, which is not allowed
during runtime because /proc is mounted read-only, add a preinit script:

	vm config preinit enable_ip_forwarding.sh'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'preinit', u'type': 'stringItem', 'optional': True, u'key': u'preinit'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config preinit', *args)
                raise ValidationError('could not understand command', args)
            preinit.shared_prefix = 'vm config preinit'
    

    
            def virtioserial(self, *args):
                '''Specify the virtio-serial ports that will be created for the VM to use.
Virtio-serial ports specified will be mapped to the VM's
/dev/virtio-port/<portname> device, where <portname> refers to the connected
unix socket on the host at $minimega_runtime/<vm_id>/virtio-serialX.

Examples:

To display current virtio-serial ports:
  vm config virtio-serial

To create three virtio-serial ports:
  vm config virtio-serial 3'''
                args = list(args)
                #validate the args
                candidates = [[{u'text': u'number of virtio-serial ports', u'type': 'stringItem', 'optional': True, u'key': u'number'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('vm config virtio-serial', *args)
                raise ValidationError('could not understand command', args)
            virtioserial.shared_prefix = 'vm config virtio-serial'
    

    

    

    
    def echo(self, *args):
        '''display input text after comment removal'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'args', u'type': 'listItem', 'optional': True, u'key': u'args'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('echo', *args)
        raise ValidationError('could not understand command', args)
    echo.shared_prefix = 'echo'
    

    
    def file(self, *args):
        '''file allows you to transfer and manage files served by minimega in the
directory set by the -filepath flag (default is 'base'/files).

To list files currently being served, issue the list command with a directory
relative to the served directory:

	file list /foo

Issuing "file list /" will list the contents of the served directory.

Files can be deleted with the delete command:

	file delete /foo

If a directory is given, the directory will be recursively deleted.

Files are transferred using the get command. When a get command is issued, the
node will begin searching for a file matching the path and name within the
mesh. If the file exists, it will be transferred to the requesting node. If
multiple different files exist with the same name, the behavior is undefined.
When a file transfer begins, control will return to minimega while the transfer
completes.

To see files that are currently being transferred, use the status command:

	file status

If a directory is specified, that directory will be recursively transferred to
the node.

You can also supply globs (wildcards) with the * operator. For example:

	file get *.qcow2'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'list,', u'type': 'choiceItem', u'options': [u'list'], 'optional': False}, {u'text': u'path', u'type': 'stringItem', 'optional': True, u'key': u'path'}], [{u'text': u'get,', u'type': 'choiceItem', u'options': [u'get'], 'optional': False}, {u'text': u'file', u'type': 'stringItem', 'optional': False, u'key': u'file'}], [{u'text': u'delete,', u'type': 'choiceItem', u'options': [u'delete'], 'optional': False}, {u'text': u'file', u'type': 'stringItem', 'optional': False, u'key': u'file'}], [{u'text': u'status,', u'type': 'choiceItem', u'options': [u'status'], 'optional': False}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('file', *args)
        raise ValidationError('could not understand command', args)
    file.shared_prefix = 'file'
    

    
    def viz(self, *args):
        '''Output the current experiment topology as a graphviz readable 'dot' file.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('viz', *args)
        raise ValidationError('could not understand command', args)
    viz.shared_prefix = 'viz'
    

    
    def disk(self, *args):
        '''Manipulate qcow disk images. Supports creating new images, snapshots of
existing images, and injecting one or more files into an existing image.

Example of creating a new disk:

	disk create qcow2 foo.qcow2 100G

The size argument is the size in bytes, or using optional suffixes "k"
(kilobyte), "M" (megabyte), "G" (gigabyte), "T" (terabyte).

Example of taking a snapshot of a disk:

	disk snapshot windows7.qc2 window7_miniccc.qc2

If the destination name is omitted, a name will be randomly generated and the
snapshot will be stored in the 'files' directory. Snapshots are always created
in the 'files' directory.

To inject files into an image:

	disk inject window7_miniccc.qc2 files "miniccc":"Program Files/miniccc

Each argument after the image should be a source and destination pair,
separated by a ':'. If the file paths contain spaces, use double quotes.
Optionally, you may specify a partition (partition 1 will be used by default):

	disk inject window7_miniccc.qc2:2 files "miniccc":"Program Files/miniccc

You can optionally specify mount arguments to use with inject. Multiple options
should be quoted. For example:

	disk inject foo.qcow2 options "-t fat -o offset=100" files foo:bar

Disk image paths are always relative to the 'files' directory. Users may also
use absolute paths if desired.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'create,', u'type': 'choiceItem', u'options': [u'create'], 'optional': False}, {u'text': u'qcow2,raw', u'type': 'choiceItem', u'options': [u'qcow2', u'raw'], 'optional': False}, {u'text': u'image name', u'type': 'stringItem', 'optional': False, u'key': u'image'}, {u'text': u'size', u'type': 'stringItem', 'optional': False, u'key': u'size'}], [{u'text': u'snapshot,', u'type': 'choiceItem', u'options': [u'snapshot'], 'optional': False}, {u'text': u'image', u'type': 'stringItem', 'optional': False, u'key': u'image'}, {u'text': u'dst image', u'type': 'stringItem', 'optional': True, u'key': u'dst'}], [{u'text': u'inject,', u'type': 'choiceItem', u'options': [u'inject'], 'optional': False}, {u'text': u'image', u'type': 'stringItem', 'optional': False, u'key': u'image'}, {u'text': u'files', u'type': 'literalItem', 'optional': False}, {u'text': u'files like /path/to/src:/path/to/dst', u'type': 'listItem', 'optional': False, u'key': u'files'}], [{u'text': u'inject,', u'type': 'choiceItem', u'options': [u'inject'], 'optional': False}, {u'text': u'image', u'type': 'stringItem', 'optional': False, u'key': u'image'}, {u'text': u'options', u'type': 'literalItem', 'optional': False}, {u'text': u'options', u'type': 'stringItem', 'optional': False, u'key': u'options'}, {u'text': u'files', u'type': 'literalItem', 'optional': False}, {u'text': u'files like /path/to/src:/path/to/dst', u'type': 'listItem', 'optional': False, u'key': u'files'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('disk', *args)
        raise ValidationError('could not understand command', args)
    disk.shared_prefix = 'disk'
    

    
    def check(self, *args):
        '''minimega maintains a list of external packages that it depends on, such as
qemu. Calling check will attempt to find each of these executables in the
avaiable path and check to make sure they meet the minimum version
requirements. Returns errors for all missing executables and all minimum
versions not met.'''
        args = list(args)
        #validate the args
        candidates = [[]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('check', *args)
        raise ValidationError('could not understand command', args)
    check.shared_prefix = 'check'
    

    
    def quit(self, *args):
        '''Quit minimega. An optional integer argument X allows deferring the quit call
for X seconds. This is useful for telling a mesh of minimega nodes to quit.

quit will not return a response to the cli, control socket, or meshage, it will
simply exit. meshage connected nodes catch this and will remove the quit node
from the mesh. External tools interfacing minimega must check for EOF on stdout
or the control socket as an indication that minimega has quit.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'delay', u'type': 'stringItem', 'optional': True, u'key': u'delay'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('quit', *args)
        raise ValidationError('could not understand command', args)
    quit.shared_prefix = 'quit'
    

    
    def nuke(self, *args):
        '''After a crash, the VM state on the machine can be difficult to recover from.
nuke attempts to kill all instances of QEMU, remove all taps and bridges, and
removes the temporary minimega state on the harddisk.

Should be run with caution.'''
        args = list(args)
        #validate the args
        candidates = [[]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('nuke', *args)
        raise ValidationError('could not understand command', args)
    nuke.shared_prefix = 'nuke'
    

    
    class log(SubCommand):
        
    
        def filter(self, *args):
            '''Control what data gets logged based on matching text. For example, to filter
out all logging messages containing the word "foo":

	log filter foo'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'filter', u'type': 'stringItem', 'optional': True, u'key': u'filter'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('log filter', *args)
            raise ValidationError('could not understand command', args)
        filter.shared_prefix = 'log filter'
    

    
        def file(self, *args):
            '''Log to a file. To disable file logging, call "clear log file".'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'file', u'type': 'stringItem', 'optional': True, u'key': u'file'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('log file', *args)
            raise ValidationError('could not understand command', args)
        file.shared_prefix = 'log file'
    

    
        def stderr(self, *args):
            '''enable or disable logging to stderr'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'true,false', u'type': 'choiceItem', u'options': [u'true', u'false'], 'optional': True}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('log stderr', *args)
            raise ValidationError('could not understand command', args)
        stderr.shared_prefix = 'log stderr'
    

    
        def level(self, *args):
            '''Set the log level to one of [debug,info,warn,error,fatal]. Log levels inherit
lower levels, so setting the level to error will also log fatal, and setting
the mode to debug will log everything.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'debug,info,warn,error,fatal', u'type': 'choiceItem', u'options': [u'debug', u'info', u'warn', u'error', u'fatal'], 'optional': True}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('log level', *args)
            raise ValidationError('could not understand command', args)
        level.shared_prefix = 'log level'
    

    

    
    def vyatta(self, *args):
        '''Define and write out vyatta router floppy disk images.

vyatta takes a number of subcommands:

- 'dhcp': Add DHCP service to a particular network by specifying the network,
default gateway, and start and stop addresses. For example, to serve dhcp on
10.0.0.0/24, with a default gateway of 10.0.0.1:

	vyatta dhcp add 10.0.0.0/24 10.0.0.1 10.0.0.2 10.0.0.254

An optional DNS argument can be used to override the nameserver. For example,
to do the same as above with a nameserver of 8.8.8.8:

	vyatta dhcp add 10.0.0.0/24 10.0.0.1 10.0.0.2 10.0.0.254 8.8.8.8

Optionally, you can specify "none" for the default gateway.

- 'interfaces': Add IPv4 addresses using CIDR notation. Optionally, 'dhcp' or
'none' may be specified. The order specified matches the order of VLANs used in
vm_net. This number of arguments must either be 0 or equal to the number of
arguments in 'interfaces6' For example:

	vyatta interfaces 10.0.0.1/24 dhcp

- 'interfaces6': Add IPv6 addresses similar to 'interfaces'. The number of
arguments must either be 0 or equal to the number of arguments in 'interfaces'.

- 'rad': Enable router advertisements for IPv6. Valid arguments are IPv6
prefixes or "none". Order matches that of interfaces6. For example:

	vyatta rad 2001::/64 2002::/64

- 'ospf': Route networks using OSPF. For example:

	vyatta ospf 10.0.0.0/24 12.0.0.0/24

- 'ospf3': Route IPv6 interfaces using OSPF3. For example:

	vyatta ospf3 eth0 eth1

- 'routes': Set static routes. Routes are specified as

	<network>,<next-hop> ...

For example:

	vyatta routes 2001::0/64,123::1 10.0.0.0/24,12.0.0.1

- 'config': Override all other options and use a specified file as the config
file. For example: vyatta config /tmp/myconfig.boot

- 'write': Write the current configuration to file. If a filename is omitted, a
random filename will be used and the file placed in the path specified by the
-filepath flag. The filename will be returned.'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'dhcp,', u'type': 'choiceItem', u'options': [u'dhcp'], 'optional': False}], [{u'text': u'dhcp,', u'type': 'choiceItem', u'options': [u'dhcp'], 'optional': False}, {u'text': u'add', u'type': 'literalItem', 'optional': False}, {u'text': u'network', u'type': 'stringItem', 'optional': False, u'key': u'network'}, {u'text': u'gateway or none', u'type': 'stringItem', 'optional': False, u'key': u'gateway'}, {u'text': u'low dhcp range', u'type': 'stringItem', 'optional': False, u'key': u'low'}, {u'text': u'high dhcp range', u'type': 'stringItem', 'optional': False, u'key': u'high'}, {u'text': u'dns server', u'type': 'stringItem', 'optional': True, u'key': u'dns'}], [{u'text': u'dhcp,', u'type': 'choiceItem', u'options': [u'dhcp'], 'optional': False}, {u'text': u'delete', u'type': 'literalItem', 'optional': False}, {u'text': u'network', u'type': 'stringItem', 'optional': False, u'key': u'network'}], [{u'text': u'interfaces,', u'type': 'choiceItem', u'options': [u'interfaces'], 'optional': False}, {u'text': u'net A.B.C.D/MASK or dhcp or none', u'type': 'listItem', 'optional': True, u'key': u'net'}], [{u'text': u'interfaces6,', u'type': 'choiceItem', u'options': [u'interfaces6'], 'optional': False}, {u'text': u'net IPv6 address/MASK or none', u'type': 'listItem', 'optional': True, u'key': u'net'}], [{u'text': u'rad,', u'type': 'choiceItem', u'options': [u'rad'], 'optional': False}, {u'text': u'prefix', u'type': 'listItem', 'optional': True, u'key': u'prefix'}], [{u'text': u'ospf,', u'type': 'choiceItem', u'options': [u'ospf'], 'optional': False}, {u'text': u'network', u'type': 'listItem', 'optional': True, u'key': u'network'}], [{u'text': u'ospf3,', u'type': 'choiceItem', u'options': [u'ospf3'], 'optional': False}, {u'text': u'network', u'type': 'listItem', 'optional': True, u'key': u'network'}], [{u'text': u'routes,', u'type': 'choiceItem', u'options': [u'routes'], 'optional': False}, {u'text': u'network and next-hop separated by comma', u'type': 'listItem', 'optional': True, u'key': u'network'}], [{u'text': u'config,', u'type': 'choiceItem', u'options': [u'config'], 'optional': False}, {u'text': u'filename', u'type': 'stringItem', 'optional': True, u'key': u'filename'}], [{u'text': u'write,', u'type': 'choiceItem', u'options': [u'write'], 'optional': False}, {u'text': u'filename', u'type': 'stringItem', 'optional': True, u'key': u'filename'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('vyatta', *args)
        raise ValidationError('could not understand command', args)
    vyatta.shared_prefix = 'vyatta'
    

    
    def write(self, *args):
        '''Write the command history to file. This is useful for handcrafting configs on
the minimega command line and then saving them for later use.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'file', u'type': 'stringItem', 'optional': False, u'key': u'file'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('write', *args)
        raise ValidationError('could not understand command', args)
    write.shared_prefix = 'write'
    

    
    def version(self, *args):
        '''display the minimega version'''
        args = list(args)
        #validate the args
        candidates = [[]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('version', *args)
        raise ValidationError('could not understand command', args)
    version.shared_prefix = 'version'
    

    
    def shell(self, *args):
        '''Execute a command under the credentials of the running user.

Commands run until they complete or error, so take care not to execute a command
that does not return.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'command', u'type': 'listItem', 'optional': False, u'key': u'command'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('shell', *args)
        raise ValidationError('could not understand command', args)
    shell.shared_prefix = 'shell'
    

    
    def vnc(self, *args):
        '''Record or playback keyboard and mouse events sent via the web interface to the
selected VM. Can also record the framebuffer for the specified VM so that a
users can watch a video of interactions with the VM.

With no arguments, vnc will list currently recording or playing VNC sessions.

If record is selected, a file will be created containing a record of mouse and
keyboard actions by the user or of the framebuffer for the VM.

If playback is selected, the specified file (created using vnc record) will be
read and processed as a sequence of time-stamped mouse/keyboard events to send
to the specified VM.'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'kb,fb', u'type': 'choiceItem', u'options': [u'kb', u'fb'], 'optional': False}, {u'text': u'record,', u'type': 'choiceItem', u'options': [u'record'], 'optional': False}, {u'text': u'host', u'type': 'stringItem', 'optional': False, u'key': u'host'}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}], [{u'text': u'kb,fb', u'type': 'choiceItem', u'options': [u'kb', u'fb'], 'optional': False}, {u'text': u'norecord,', u'type': 'choiceItem', u'options': [u'norecord'], 'optional': False}, {u'text': u'host', u'type': 'stringItem', 'optional': False, u'key': u'host'}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}], [{u'text': u'playback,', u'type': 'choiceItem', u'options': [u'playback'], 'optional': False}, {u'text': u'host', u'type': 'stringItem', 'optional': False, u'key': u'host'}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}], [{u'text': u'noplayback,', u'type': 'choiceItem', u'options': [u'noplayback'], 'optional': False}, {u'text': u'host', u'type': 'stringItem', 'optional': False, u'key': u'host'}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('vnc', *args)
        raise ValidationError('could not understand command', args)
    vnc.shared_prefix = 'vnc'
    

    
    def deploy(self, *args):
        '''deploy copies and runs minimega on remote nodes, facilitating the deployment of
minimega to a cluster. By default, deploy will launch minimega with the same
flags used when starting this minimega, and add the -nostdin flag so that the
remote minimega can be backgrounded. For example, to launch minimega on nodes
kn1 and kn2:

	deploy launch kn[1-2]

deploy uses scp/ssh to copy and run minimega. By default, minimega will attempt
to login to remote nodes using the current user. This can be changed by
providing a username. If using a different username, you can optionally specify
the use of sudo when launching minimega (you typically need to run minimega as
root).

In order to override the flags passed to remote minimega instances, provide
flags with 'deploy flags'. For example:

	deploy flags -base=/opt/minimega -level=debug'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'launch,', u'type': 'choiceItem', u'options': [u'launch'], 'optional': False}, {u'text': u'hosts', u'type': 'stringItem', 'optional': False, u'key': u'hosts'}], [{u'text': u'launch,', u'type': 'choiceItem', u'options': [u'launch'], 'optional': False}, {u'text': u'hosts', u'type': 'stringItem', 'optional': False, u'key': u'hosts'}, {u'text': u'user', u'type': 'stringItem', 'optional': False, u'key': u'user'}, {u'text': u'sudo,', u'type': 'choiceItem', u'options': [u'sudo'], 'optional': True}], [{u'text': u'flags,', u'type': 'choiceItem', u'options': [u'flags'], 'optional': False}, {u'text': u'minimega flags', u'type': 'listItem', 'optional': True, u'key': u'minimega'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('deploy', *args)
        raise ValidationError('could not understand command', args)
    deploy.shared_prefix = 'deploy'
    

    
    def read(self, *args):
        '''Read a command file and execute it. This has the same behavior as if you typed
the file in manually except that it stops after the first error.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'file', u'type': 'stringItem', 'optional': False, u'key': u'file'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('read', *args)
        raise ValidationError('could not understand command', args)
    read.shared_prefix = 'read'
    

    
    def capture(self, *args):
        '''Capture experiment data including netflow and PCAP. Netflow capture obtains
netflow data from any local openvswitch switch, and can write to file, another
socket, or both. Netflow data can be written out in raw or ascii format, and
file output can be compressed on the fly. Multiple netflow writers can be
configured.

PCAP capture can be from a bridge or VM interface. No filters are applied, and
all data seen on that interface is captured to file.

For example, to capture netflow data on bridge mega_bridge to file in ascii
mode and with gzip compression:

	capture netflow mega_bridge file foo.netflow ascii gzip

You can change the active flow timeout with:

	capture netflow mega_bridge timeout <timeout>

With <timeout> in seconds.

To capture pcap on bridge 'foo' to file 'foo.pcap':

	capture pcap bridge foo foo.pcap

To capture pcap on VM 'foo' to file 'foo.pcap', using the 2nd interface on that
VM:

	capture pcap vm foo 0 foo.pcap

When run without arguments, capture prints all running captures. To stop a
capture, use the delete commands:

	capture netflow delete <id>
	capture pcap delete <id>

To stop all captures of a particular kind, replace id with "all". To stop all
capture of all types, use "clear capture".'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}, {u'text': u'timeout,', u'type': 'choiceItem', u'options': [u'timeout'], 'optional': False}, {u'text': u'timeout', u'type': 'stringItem', 'optional': True, u'key': u'timeout'}], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}, {u'text': u'bridge,', u'type': 'choiceItem', u'options': [u'bridge'], 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}, {u'text': u'bridge,', u'type': 'choiceItem', u'options': [u'bridge'], 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'file,', u'type': 'choiceItem', u'options': [u'file'], 'optional': False}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}, {u'text': u'bridge,', u'type': 'choiceItem', u'options': [u'bridge'], 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'file,', u'type': 'choiceItem', u'options': [u'file'], 'optional': False}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}, {u'text': u'raw,ascii', u'type': 'choiceItem', u'options': [u'raw', u'ascii'], 'optional': False}, {u'text': u'gzip', u'type': 'stringItem', 'optional': True, u'key': u'gzip'}], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}, {u'text': u'bridge,', u'type': 'choiceItem', u'options': [u'bridge'], 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'socket,', u'type': 'choiceItem', u'options': [u'socket'], 'optional': False}, {u'text': u'tcp,udp', u'type': 'choiceItem', u'options': [u'tcp', u'udp'], 'optional': False}, {u'text': u'hostname:port', u'type': 'stringItem', 'optional': False, u'key': u'hostname:port'}, {u'text': u'raw,ascii', u'type': 'choiceItem', u'options': [u'raw', u'ascii'], 'optional': False}], [{u'text': u'netflow,', u'type': 'choiceItem', u'options': [u'netflow'], 'optional': False}, {u'text': u'delete,', u'type': 'choiceItem', u'options': [u'delete'], 'optional': False}, {u'text': u'id or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}], [{u'text': u'pcap,', u'type': 'choiceItem', u'options': [u'pcap'], 'optional': False}], [{u'text': u'pcap,', u'type': 'choiceItem', u'options': [u'pcap'], 'optional': False}, {u'text': u'bridge', u'type': 'literalItem', 'optional': False}, {u'text': u'bridge', u'type': 'stringItem', 'optional': False, u'key': u'bridge'}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}], [{u'text': u'pcap,', u'type': 'choiceItem', u'options': [u'pcap'], 'optional': False}, {u'text': u'vm', u'type': 'literalItem', 'optional': False}, {u'text': u'vm id or name', u'type': 'stringItem', 'optional': False, u'key': u'vm'}, {u'text': u'interface index', u'type': 'stringItem', 'optional': False, u'key': u'interface'}, {u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}], [{u'text': u'pcap,', u'type': 'choiceItem', u'options': [u'pcap'], 'optional': False}, {u'text': u'delete,', u'type': 'choiceItem', u'options': [u'delete'], 'optional': False}, {u'text': u'id or all', u'type': 'stringItem', 'optional': False, u'key': u'id'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('capture', *args)
        raise ValidationError('could not understand command', args)
    capture.shared_prefix = 'capture'
    

    
    def host(self, *args):
        '''report information about the host'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'name,', u'type': 'choiceItem', u'options': [u'name'], 'optional': False}], [{u'text': u'memused,', u'type': 'choiceItem', u'options': [u'memused'], 'optional': False}], [{u'text': u'memtotal,', u'type': 'choiceItem', u'options': [u'memtotal'], 'optional': False}], [{u'text': u'load,', u'type': 'choiceItem', u'options': [u'load'], 'optional': False}], [{u'text': u'bandwidth,', u'type': 'choiceItem', u'options': [u'bandwidth'], 'optional': False}], [{u'text': u'cpus,', u'type': 'choiceItem', u'options': [u'cpus'], 'optional': False}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('host', *args)
        raise ValidationError('could not understand command', args)
    host.shared_prefix = 'host'
    

    
    class mesh(SubCommand):
        
    
        def status(self, *args):
            '''display a short status report of the mesh'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh status', *args)
            raise ValidationError('could not understand command', args)
        status.shared_prefix = 'mesh status'
    

    
        def dial(self, *args):
            '''attempt to connect this node to another node'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'hostname', u'type': 'stringItem', 'optional': False, u'key': u'hostname'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh dial', *args)
            raise ValidationError('could not understand command', args)
        dial.shared_prefix = 'mesh dial'
    

    
        def degree(self, *args):
            '''view or set the current degree for this mesh node'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'degree', u'type': 'stringItem', 'optional': True, u'key': u'degree'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh degree', *args)
            raise ValidationError('could not understand command', args)
        degree.shared_prefix = 'mesh degree'
    

    
        def list(self, *args):
            '''display the mesh adjacency list'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh list', *args)
            raise ValidationError('could not understand command', args)
        list.shared_prefix = 'mesh list'
    

    
        def send(self, *args):
            '''Send a command to one or more connected clients. For example, to get the
vm info from nodes kn1 and kn2:

	mesh send kn[1-2] vm info

You can use 'all' to send a command to all connected clients.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'vms or all', u'type': 'stringItem', 'optional': False, u'key': u'vms'}, {u'text': u'command', u'type': 'commandItem', 'optional': False, u'key': u'command'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh send', *args)
            raise ValidationError('could not understand command', args)
        send.shared_prefix = 'mesh send'
    

    
        def hangup(self, *args):
            '''disconnect from a client'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'hostname', u'type': 'stringItem', 'optional': False, u'key': u'hostname'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh hangup', *args)
            raise ValidationError('could not understand command', args)
        hangup.shared_prefix = 'mesh hangup'
    

    
        def timeout(self, *args):
            '''View or set the timeout on sending mesh commands.

When a mesh command is issued, if a response isn't sent within mesh_timeout
seconds, the command will be dropped and any future response will be discarded.
Note that this does not cancel the outstanding command - the node receiving the
command may still complete - but rather this node will stop waiting on a
response.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'timeout', u'type': 'stringItem', 'optional': True, u'key': u'timeout'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh timeout', *args)
            raise ValidationError('could not understand command', args)
        timeout.shared_prefix = 'mesh timeout'
    

    
        def dot(self, *args):
            '''Output a graphviz formatted dot file representing the connected topology.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'filename', u'type': 'stringItem', 'optional': False, u'key': u'filename'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('mesh dot', *args)
            raise ValidationError('could not understand command', args)
        dot.shared_prefix = 'mesh dot'
    

    

    
    def web(self, *args):
        '''Launch the minimega webserver. Running web starts the HTTP server whose port
cannot be changed once started. The default port is 9001. To run the server on
a different port, run:

	web 10000

The webserver requires several resources found in misc/web in the repo. By
default, it looks in $PWD/misc/web for these resources. If you are running
minimega from a different location, you can specify a different path using:

	web root <path/to/web/dir>

You can also set the port when starting web with an alternative root directory:

	web root <path/to/web/dir> 10000

NOTE: If you start the webserver with an invalid root, you can safely re-run
"web root" to update it. You cannot, however, change the server's port.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'port', u'type': 'stringItem', 'optional': True, u'key': u'port'}], [{u'text': u'root', u'type': 'literalItem', 'optional': False}, {u'text': u'path', u'type': 'stringItem', 'optional': False, u'key': u'path'}, {u'text': u'port', u'type': 'stringItem', 'optional': True, u'key': u'port'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('web', *args)
        raise ValidationError('could not understand command', args)
    web.shared_prefix = 'web'
    

    
    def background(self, *args):
        '''Execute a command under the credentials of the running user.

Commands run in the background and control returns immediately. Any output is
logged at the "info" level.'''
        args = list(args)
        #validate the args
        candidates = [[{u'text': u'command', u'type': 'listItem', 'optional': False, u'key': u'command'}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('background', *args)
        raise ValidationError('could not understand command', args)
    background.shared_prefix = 'background'
    

    
    def optimize(self, *args):
        '''Enable or disable several virtualization optimizations, including Kernel
Samepage Merging, CPU affinity for VMs, and the use of hugepages.

To enable/disable Kernel Samepage Merging (KSM):
	optimize ksm [true,false]

To enable hugepage support:
	optimize hugepages </path/to/hugepages_mount>

To disable hugepage support:
	clear optimize hugepages

To enable/disable CPU affinity support:
	optimize affinity [true,false]

To set a CPU set filter for the affinity scheduler, for example (to use only
CPUs 1, 2-20):
	optimize affinity filter [1,2-20]

To clear a CPU set filter:
	clear optimize affinity filter

To view current CPU affinity mappings:
	optimize affinity

To disable all optimizations see "clear optimize".'''
        args = list(args)
        #validate the args
        candidates = [[], [{u'text': u'affinity,', u'type': 'choiceItem', u'options': [u'affinity'], 'optional': False}, {u'text': u'filter,', u'type': 'choiceItem', u'options': [u'filter'], 'optional': False}, {u'text': u'filter', u'type': 'stringItem', 'optional': False, u'key': u'filter'}], [{u'text': u'affinity,', u'type': 'choiceItem', u'options': [u'affinity'], 'optional': False}, {u'text': u'true,false', u'type': 'choiceItem', u'options': [u'true', u'false'], 'optional': True}], [{u'text': u'hugepages,', u'type': 'choiceItem', u'options': [u'hugepages'], 'optional': False}, {u'text': u'path', u'type': 'stringItem', 'optional': True, u'key': u'path'}], [{u'text': u'ksm,', u'type': 'choiceItem', u'options': [u'ksm'], 'optional': False}, {u'text': u'true,false', u'type': 'choiceItem', u'options': [u'true', u'false'], 'optional': True}]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('optimize', *args)
        raise ValidationError('could not understand command', args)
    optimize.shared_prefix = 'optimize'
    

    
    class clear(SubCommand):
        
    
        def capture(self, *args):
            '''Resets state for captures. See "help capture" for more information.'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'netflow,pcap', u'type': 'choiceItem', u'options': [u'netflow', u'pcap'], 'optional': True}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear capture', *args)
            raise ValidationError('could not understand command', args)
        capture.shared_prefix = 'clear capture'
    

    
        def tap(self, *args):
            '''Reset state for taps. See "help tap" for more information.'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear tap', *args)
            raise ValidationError('could not understand command', args)
        tap.shared_prefix = 'clear tap'
    

    
        def log(self, *args):
            '''Resets state for logging. See "help log ..." for more information.'''
            args = list(args)
            #validate the args
            candidates = [[], [{u'text': u'file,', u'type': 'choiceItem', u'options': [u'file'], 'optional': False}], [{u'text': u'level,', u'type': 'choiceItem', u'options': [u'level'], 'optional': False}], [{u'text': u'stderr,', u'type': 'choiceItem', u'options': [u'stderr'], 'optional': False}], [{u'text': u'filter,', u'type': 'choiceItem', u'options': [u'filter'], 'optional': False}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear log', *args)
            raise ValidationError('could not understand command', args)
        log.shared_prefix = 'clear log'
    

    
        class deploy(SubCommand):
        
    
            def flags(self, *args):
                '''Reset the deploy flags to their default value, which is equal to the launch
flags used when launching minimega.'''
                args = list(args)
                #validate the args
                candidates = [[]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('clear deploy flags', *args)
                raise ValidationError('could not understand command', args)
            flags.shared_prefix = 'clear deploy flags'
    

    

    
        def cc(self, *args):
            '''Resets state for the command and control infrastructure provided by minimega.
See "help cc" for more information.'''
            args = list(args)
            #validate the args
            candidates = [[], [{u'text': u'commands,', u'type': 'choiceItem', u'options': [u'commands'], 'optional': False}], [{u'text': u'filter,', u'type': 'choiceItem', u'options': [u'filter'], 'optional': False}], [{u'text': u'prefix,', u'type': 'choiceItem', u'options': [u'prefix'], 'optional': False}], [{u'text': u'responses,', u'type': 'choiceItem', u'options': [u'responses'], 'optional': False}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear cc', *args)
            raise ValidationError('could not understand command', args)
        cc.shared_prefix = 'clear cc'
    

    
        class vm(SubCommand):
        
    
            def tag(self, *args):
                '''Clears one, many, or all tags from a virtual machine.

Clear the tag "foo" from VM 0:

        clear vm tag 0 foo

Clear the tag "foo" from all VMs:

        clear vm tag all foo

Clear all tags from VM 0:

        clear vm tag 0

Clear all tags from all VMs:

        clear vm tag all'''
                args = list(args)
                #validate the args
                candidates = [[], [{u'text': u'target', u'type': 'stringItem', 'optional': False, u'key': u'target'}, {u'text': u'tag', u'type': 'stringItem', 'optional': True, u'key': u'tag'}]]
                for candidate in candidates:
                    argNum = 0
                    try:
                        for arg in candidate:
                            if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                if self.mm._debug:
                                    print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            if arg['type'] == 'listItem':
                                if not isinstance(args[argNum], list):
                                    if self.mm._debug:
                                        print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = ' '.join(map(str, args[argNum]))
                            if arg['type'] == 'commandItem':
                                if not isinstance(args[argNum], Command):
                                    if self.mm._debug:
                                        print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                args[argNum] = serializeCommand(args[argNum])
                            if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                if self.mm._debug:
                                    print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                break
                            argNum += 1
                    except IndexError:
                        if not candidate[argNum]['optional']:
                            if self.mm._debug:
                                print('"{}" required but not provided'.format(arg['text']))
                            continue #skip to next candidate
                    if not args[argNum:]:
                        #passed validation, exact match for this candidate
                        #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                        return self.mm._send('clear vm tag', *args)
                raise ValidationError('could not understand command', args)
            tag.shared_prefix = 'clear vm tag'
    

    
            class config(SubCommand):
        
    
                def tag(self, *args):
                    '''Remove tags in the same manner as "clear vm tag".'''
                    args = list(args)
                    #validate the args
                    candidates = [[{u'text': u'key', u'type': 'stringItem', 'optional': True, u'key': u'key'}]]
                    for candidate in candidates:
                        argNum = 0
                        try:
                            for arg in candidate:
                                if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                                    if self.mm._debug:
                                        print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                    break
                                if arg['type'] == 'listItem':
                                    if not isinstance(args[argNum], list):
                                        if self.mm._debug:
                                            print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                        break
                                    args[argNum] = ' '.join(map(str, args[argNum]))
                                if arg['type'] == 'commandItem':
                                    if not isinstance(args[argNum], Command):
                                        if self.mm._debug:
                                            print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                        break
                                    args[argNum] = serializeCommand(args[argNum])
                                if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                                    if self.mm._debug:
                                        print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                                    break
                                argNum += 1
                        except IndexError:
                            if not candidate[argNum]['optional']:
                                if self.mm._debug:
                                    print('"{}" required but not provided'.format(arg['text']))
                                continue #skip to next candidate
                        if not args[argNum:]:
                            #passed validation, exact match for this candidate
                            #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                            return self.mm._send('clear vm config tag', *args)
                    raise ValidationError('could not understand command', args)
                tag.shared_prefix = 'clear vm config tag'
    

    

    

    
        def vyatta(self, *args):
            '''Resets state for vyatta. See "help vyatta" for more information.'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear vyatta', *args)
            raise ValidationError('could not understand command', args)
        vyatta.shared_prefix = 'clear vyatta'
    

    
        def vnc(self, *args):
            '''Resets the state for VNC recordings. See "help vnc" for more information.'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear vnc', *args)
            raise ValidationError('could not understand command', args)
        vnc.shared_prefix = 'clear vnc'
    

    
        def optimize(self, *args):
            '''Resets state for virtualization optimizations. See "help optimize" for more
information.'''
            args = list(args)
            #validate the args
            candidates = [[], [{u'text': u'affinity,', u'type': 'choiceItem', u'options': [u'affinity'], 'optional': False}, {u'text': u'filter,', u'type': 'choiceItem', u'options': [u'filter'], 'optional': True}], [{u'text': u'hugepages,', u'type': 'choiceItem', u'options': [u'hugepages'], 'optional': False}], [{u'text': u'ksm,', u'type': 'choiceItem', u'options': [u'ksm'], 'optional': False}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear optimize', *args)
            raise ValidationError('could not understand command', args)
        optimize.shared_prefix = 'clear optimize'
    

    
        def history(self, *args):
            '''Reset the command history. See "help history" for more information.'''
            args = list(args)
            #validate the args
            candidates = [[]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('clear history', *args)
            raise ValidationError('could not understand command', args)
        history.shared_prefix = 'clear history'
    

    

    
    class dnsmasq(SubCommand):
        
    
        def configure(self, *args):
            '''Configuration options for running dnsmasq instances. Define a static IP
allocation, specify a hostname->IP mapping for DNS, or set DHCP options.

To list all existing static IP allocations on the first running dnsmasq
server, do the following:

	dnsmasq configure 0 ip

To set up a static IP allocation for a VM with the MAC address
00:11:22:33:44:55:

	dnsmasq configure 0 ip 00:11:22:33:44:55 172.17.0.50

To see DNS entries:

	dnsmasq configure 0 dns

To add a DNS entry:

	dnsmasq configure 0 dns 172.17.0.50 example.com

To see a list of all DHCP options:

	dnsmasq configure 0 options

To add a DHCP option:

	dnsmasq configure 0 options option:dns-server,172.17.0.254'''
            args = list(args)
            #validate the args
            candidates = [[{u'text': u'ID', u'type': 'stringItem', 'optional': False, u'key': u'ID'}, {u'text': u'ip,', u'type': 'choiceItem', u'options': [u'ip'], 'optional': False}], [{u'text': u'ID', u'type': 'stringItem', 'optional': False, u'key': u'ID'}, {u'text': u'ip,', u'type': 'choiceItem', u'options': [u'ip'], 'optional': False}, {u'text': u'mac address', u'type': 'stringItem', 'optional': False, u'key': u'mac'}, {u'text': u'ip', u'type': 'stringItem', 'optional': False, u'key': u'ip'}], [{u'text': u'ID', u'type': 'stringItem', 'optional': False, u'key': u'ID'}, {u'text': u'dns,', u'type': 'choiceItem', u'options': [u'dns'], 'optional': False}], [{u'text': u'ID', u'type': 'stringItem', 'optional': False, u'key': u'ID'}, {u'text': u'dns,', u'type': 'choiceItem', u'options': [u'dns'], 'optional': False}, {u'text': u'ip', u'type': 'stringItem', 'optional': False, u'key': u'ip'}, {u'text': u'hostname', u'type': 'stringItem', 'optional': False, u'key': u'hostname'}], [{u'text': u'ID', u'type': 'stringItem', 'optional': False, u'key': u'ID'}, {u'text': u'options,', u'type': 'choiceItem', u'options': [u'options'], 'optional': False}], [{u'text': u'ID', u'type': 'stringItem', 'optional': False, u'key': u'ID'}, {u'text': u'options,', u'type': 'choiceItem', u'options': [u'options'], 'optional': False}, {u'text': u'optionstring', u'type': 'stringItem', 'optional': False, u'key': u'optionstring'}]]
            for candidate in candidates:
                argNum = 0
                try:
                    for arg in candidate:
                        if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                            if self.mm._debug:
                                print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        if arg['type'] == 'listItem':
                            if not isinstance(args[argNum], list):
                                if self.mm._debug:
                                    print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = ' '.join(map(str, args[argNum]))
                        if arg['type'] == 'commandItem':
                            if not isinstance(args[argNum], Command):
                                if self.mm._debug:
                                    print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                                break
                            args[argNum] = serializeCommand(args[argNum])
                        if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                            if self.mm._debug:
                                print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                            break
                        argNum += 1
                except IndexError:
                    if not candidate[argNum]['optional']:
                        if self.mm._debug:
                            print('"{}" required but not provided'.format(arg['text']))
                        continue #skip to next candidate
                if not args[argNum:]:
                    #passed validation, exact match for this candidate
                    #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                    return self.mm._send('dnsmasq configure', *args)
            raise ValidationError('could not understand command', args)
        configure.shared_prefix = 'dnsmasq configure'
    

    

    
    def debug(self, *args):
        '''display internal debug information'''
        args = list(args)
        #validate the args
        candidates = [[]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('debug', *args)
        raise ValidationError('could not understand command', args)
    debug.shared_prefix = 'debug'
    

    
    def history(self, *args):
        '''history displays a list of all the commands that have been invoked since
minimega started on this host, or since the last time the history was cleared.
History includes only valid commands and comments. Invalid lines and blank
lines are not recorded. There are some commands that interact differently with
history, namely read. Instead of recording the "read" command in the history,
minimega records all the valid commands executed from the read file in the
history. This allows the full execution history to be listed using history.'''
        args = list(args)
        #validate the args
        candidates = [[]]
        for candidate in candidates:
            argNum = 0
            try:
                for arg in candidate:
                    if arg['type'] == 'stringItem' and not _isstr(args[argNum]):
                        if self.mm._debug:
                            print('expected string for "{}", received {}'.format(arg['text'], type(args[argNum])))
                        break
                    if arg['type'] == 'listItem':
                        if not isinstance(args[argNum], list):
                            if self.mm._debug:
                                print('expected list for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = ' '.join(map(str, args[argNum]))
                    if arg['type'] == 'commandItem':
                        if not isinstance(args[argNum], Command):
                            if self.mm._debug:
                                print('expected Command object for "{}", received {}'.format(arg['text'], type(args[argNum])))
                            break
                        args[argNum] = serializeCommand(args[argNum])
                    if arg['type'] == 'choiceItem' and args[argNum] not in arg['options']:
                        if self.mm._debug:
                            print('expected one of "{}" for "{}", received {}'.format(arg['options'], arg['text'], args[argNum]))
                        break
                    argNum += 1
            except IndexError:
                if not candidate[argNum]['optional']:
                    if self.mm._debug:
                        print('"{}" required but not provided'.format(arg['text']))
                    continue #skip to next candidate
            if not args[argNum:]:
                #passed validation, exact match for this candidate
                #if we break in the loop above, argNum doesn't get incremented and args[argNum:] will never be empty, so we don't need a "valid" flag
                return self.mm._send('history', *args)
        raise ValidationError('could not understand command', args)
    history.shared_prefix = 'history'
    

